@@ -53,10 +53,11 @@ decideAndJump <- nimbleFunction(
     setup = function(model, mvSaved, target, UNUSED) {    ## should remove UNUSED argument, after next release of nimbleSMC -DT July 2024
         ccList <- mcmc_determineCalcAndCopyNodes(model, target)
         copyNodesDeterm <- ccList$copyNodesDeterm; copyNodesStoch <- ccList$copyNodesStoch  # not used: calcNodes, calcNodesNoSelf
+        targetNames <- createNamesString(target)
     },
     run = function(modelLP1 = double(), modelLP0 = double(), propLP1 = double(), propLP0 = double()) {
         ## Check each one individually to catch case like `3 - Inf`.
-        logMHR <- checkLogProb(modelLP1) - checkLogProb(modelLP0) - checkLogProb(propLP1) + checkLogProb(propLP0)
+        logMHR <- checkLogProb(modelLP1, targetNames) - checkLogProb(modelLP0, targetNames) - checkLogProb(propLP1, targetNames) + checkLogProb(propLP0, targetNames)
         jump <- decide(logMHR)
         if(jump) {
             nimCopy(from = model, to = mvSaved, row = 1, nodes = target, logProb = TRUE)
@@ -72,11 +73,11 @@ decideAndJump <- nimbleFunction(
     }
 )
 
-checkLogProb <- function(logProb) {
+checkLogProb <- function(logProb, target) {
    if(is.na(logProb))
        return(-Inf)
    if(logProb == Inf)
-         print("MCMC sampling encountered a log probability density value of infinity. Results of sampling may not be valid.")
+         cat("MCMC sampling of ", target, " encountered a log probability density value of infinity. Results of sampling may not be valid.\n")
    return(logProb)
 }
 
@@ -266,7 +267,8 @@ mcmc_generateControlListArgument <- function(control, controlDefaults) {
 
 
 
-mcmc_listContentsToStr <- function(ls, displayControlDefaults=FALSE, displayNonScalars=FALSE, displayConjugateDependencies=FALSE) {
+mcmc_listContentsToStr <- function(ls, displayControlDefaults=FALSE, displayNonScalars=FALSE, displayConjugateDependencies=FALSE,
+                                   removeCfunctions = TRUE, removeLengthZero = TRUE) {
     ##if(any(unlist(lapply(ls, is.function)))) warning('probably provided wrong type of function argument')
     if(!displayConjugateDependencies) {
         if(grepl('^conjugate_d', names(ls)[1])) ls <- ls[1]    ## for conjugate samplers, remove all 'dep_dnorm', etc, control elements (don't print them!)
@@ -287,7 +289,7 @@ mcmc_listContentsToStr <- function(ls, displayControlDefaults=FALSE, displayNonS
     for(i in seq_along(ls)) {
         controlName <- names(ls)[i]
         controlValue <- ls[[i]]
-        if(length(controlValue) == 0) next   ## remove length 0
+        if(length(controlValue) == 0 && removeLengthZero) next   ## remove length 0
         ##if(!displayControlDefaults)
         ##    if(controlName %in% names(defaultOptions))   ## skip default control values
         ##        if(identical(controlValue, defaultOptions[[controlName]])) next
@@ -305,7 +307,7 @@ mcmc_listContentsToStr <- function(ls, displayControlDefaults=FALSE, displayNonS
     ##if(length(ls2) == 1)
     ##    str <- paste0(str, ', default')
     str <- gsub('\"', '', str)
-    str <- gsub('c\\((.*?)\\)', '\\1', str)
+    if(removeCfunctions)   str <- gsub('c\\((.*?)\\)', '\\1', str)
     return(str)
 }
 
@@ -422,7 +424,7 @@ mcmc_checkWAICmonitors <- function(model, monitors, dataNodes) {
 }
 
 
-mcmc_createModelObject <- function(model, inits, nchains, setSeed, code, constants, data, dimensions, check, buildDerivs = FALSE) {
+mcmc_createModelObject <- function(model, inits, nchains, setSeed, code, constants, data, dimensions, check, buildDerivs = FALSE, userEnv) {
     ## create the Rmodel object using arguments provided to nimbleMCMC
     if(missing(model)) {  ## model object not provided
         if(!missing(inits)) {
@@ -434,8 +436,8 @@ mcmc_createModelObject <- function(model, inits, nchains, setSeed, code, constan
             } else if(is.list(inits) && (length(inits) > 0) && is.list(inits[[1]])) {
                 theseInits <- inits[[1]]
             } else theseInits <- inits
-            Rmodel    <- nimbleModel(code, constants, data, theseInits, dimensions = dimensions, check = check, buildDerivs = buildDerivs)    ## inits provided
-        } else Rmodel <- nimbleModel(code, constants, data,             dimensions = dimensions, check = check, buildDerivs = buildDerivs)    ## inits not provided
+            Rmodel    <- nimbleModel(code, constants, data, theseInits, dimensions = dimensions, check = check, buildDerivs = buildDerivs, userEnv = userEnv)    ## inits provided
+        } else Rmodel <- nimbleModel(code, constants, data,             dimensions = dimensions, check = check, buildDerivs = buildDerivs, userEnv = userEnv)    ## inits not provided
     } else {              ## model object provided
         if(!is.model(model)) stop('model argument must be a NIMBLE model object')
         Rmodel <- if(is.Rmodel(model)) model else model$Rmodel
@@ -550,6 +552,43 @@ mcmc_checkTargetAD <- function(model, targetNodes, samplerType) {
              paste0(dists[!ADok], collapse = ', '))
 }
 
+createNamesString <- function(target) {
+    if(length(target) == 1) return(target)
+    if(length(target) > 4) target <- c(target[1:4], "...")
+    return(paste0("{", paste(target, collapse = ', '), "}"))
+}
+
+# This is function which builds a new MCMCconf from an old MCMCconf
+# This is required to be able to a new C-based MCMC without recompiling
+makeNewConfFromOldConf <- function(oldMCMCconf){
+    newMCMCconf <- configureMCMC(oldMCMCconf$model, nodes = NULL, print = FALSE)
+    newMCMCconf$monitors <- oldMCMCconf$monitors
+    newMCMCconf$monitors2 <- oldMCMCconf$monitors2
+    newMCMCconf$thin <- oldMCMCconf$thin
+    newMCMCconf$thin2 <- oldMCMCconf$thin2
+    newMCMCconf$samplerConfs <- oldMCMCconf$samplerConfs
+    newMCMCconf$samplerExecutionOrder <- oldMCMCconf$samplerExecutionOrder
+    newMCMCconf$controlDefaults <- oldMCMCconf$controlDefaults
+    ##newMCMCconf$namedSamplerLabelMaker <- oldMCMCconf$namedSamplerLabelMaker  ## usage long since deprecated (Dec 2020)
+    newMCMCconf$mvSamples1Conf <- oldMCMCconf$mvSamples1Conf
+    newMCMCconf$mvSamples2Conf <- oldMCMCconf$mvSamples2Conf
+    return(newMCMCconf)
+}
+
+
+newSpacesFunction <- function(m) {
+    log10max <- floor(log10(m))
+    function(i) paste0(rep(' ', log10max-floor(log10(i))), collapse = '')
+}
+
+
+getBaseClassName <- function(nf) {
+    baseName <- environment(environment(nf)$contains)$className
+    if(is.null(baseName)) warning('cannot find base class name')
+    return(baseName)
+}
+
+
 
 
 
