@@ -91,6 +91,7 @@ toEigenCalls <- c(
     makeCallList(c('nim_IsNA',
                    'nim_IsNaN'), 'toEigenScalarRecurse'),
     makeCallList(c('nimArr_dmnorm_chol',
+                   'nimArr_dmnorm_inv_ld',
                    'nimArr_dmvt_chol',
                    'nimArr_dlkj_corr_cholesky',
                    'nimArr_dwish_chol',
@@ -100,6 +101,7 @@ toEigenCalls <- c(
                    'nimArr_dinterval',
                    'nimArr_ddirch'), 'toEigenScalarRecurse'),
     makeCallList(c('nimArr_rmnorm_chol',
+                   'nimArr_rmnorm_inv_ld',
                    'nimArr_rmvt_chol',
                    'nimArr_rlkj_corr_cholesky',
                    'nimArr_rwish_chol',
