@@ -3,17 +3,17 @@
  * Copyright (C) 2014-2017 Perry de Valpine, Christopher Paciorek,
  * Daniel Turek, Clifford Anderson-Bergman, Nick Michaud, Fritz Obermeyer,
  * Duncan Temple Lang.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, a copy is available at
  * https://www.R-project.org/Licenses/
@@ -242,11 +242,11 @@ class NimOptimProblem_Fun_Grad_Hess : public NimOptimProblem_Fun_Grad<Fn, Gr> {
   protected:
   virtual void hessian_callback() {
     const int n = NimOptimProblem::par_.dimSize(0);
-    NimOptimProblem::ans_hessian_ = he_();
+    NimOptimProblem::ans_hessian_ = he_(NimOptimProblem::par_);
     double* parscale = NimOptimProblem::working_parscale.getPtr();
     for(int i = 0; i < n; ++i) {
       for(int j = 0; j < n; ++j) {
-        NimOptimProblem::ans_hessian_[i,j] *= parscale[i] * parscale[j];
+        NimOptimProblem::ans_hessian_(i,j) *= parscale[i] * parscale[j];
       }
     }
   }
