@@ -1271,6 +1271,37 @@ test_that("message given if same parameter is generated by macros multiple times
   nimbleOptions(verbose = FALSE)
 })
 
+test_that("error when macro has use3pieces = TRUE but code is not an assignment", {
+    testMacro <- nimble:::buildMacro(
+        function(stoch, LHS, RHSarg, modelInfo, .env) {
+            ans <- substitute(
+                OP(LHS, dnorm(RHSarg, 1)),
+                list(LHS = LHS, RHSarg = RHSarg,
+                     OP = if(stoch) as.name('~') else as.name('<-'))
+            )
+            list(code = ans, modelInfo=modelInfo)
+        },
+        use3pieces = TRUE, ## default
+        unpackArgs = TRUE  ## default
+    )
+
+    # Code without LHS/assigment
+    code <- quote(testMacro(y[1]))
+
+    expect_error(
+      testMacro$process(code, modelInfo=list(), .env=environment()),
+      "This macro must be used as part of an assignment"
+    )
+
+    code <- quote(~testMacro(y[1]))
+
+    expect_error(
+      testMacro$process(code, modelInfo=list(), .env=environment()),
+      "This macro must be used as part of an assignment"
+    )
+
+})
+
 options(warn = RwarnLevel)
 nimbleOptions(verbose = nimbleVerboseSetting)
 
